
with Interfaces.C_Streams;              use Interfaces.C_Streams;

package body Allegro.Files is

    function Find_Allegro_Resource( resource   : String;
                                    ext,
                                    datafile,
                                    objectname,
                                    envvar,
                                    subdir     : String := "" ) return String is

        function C_Find_Allegro_Resource( dest       : Address;
                                          resource,
                                          ext,
                                          datafile,
                                          objectname,
                                          envvar,
                                          subdir     : C.char_array;
                                          size       : Integer ) return Integer;
        pragma Import( C, C_Find_Allegro_Resource, "find_allegro_resource" );

        result : aliased C.char_array(C.size_t(1)..C.size_t(max_path_len));
    begin
        if C_Find_Allegro_Resource( result'Address, To_C( resource ),
                                    To_C( ext ), To_C( datafile ), To_C( objectname ),
                                    To_C( envvar ), To_C( subdir ), result'Length ) = 0 then
            return To_Ada( result );
        else
            return "";
        end if;
    end Find_Allegro_Resource;

    ----------------------------------------------------------------------------

    procedure Pack_FClose0( f : in out A_Packfile ) is
    begin
        if Pack_FClose( f ) = 0 then
            null;
        end if;
        f := null;
    end Pack_FClose0;

    ----------------------------------------------------------------------------

    function Pack_FEOF( f : not null A_Packfile ) return Boolean is

        function C_Pack_FEOF( f : A_Packfile ) return Integer;
        pragma Import( C, C_Pack_FEOF, "pack_feof" );

    begin
        return C_Pack_FEOF( f ) /= 0;
    end Pack_FEOF;

    ----------------------------------------------------------------------------

    function Pack_FError( f : not null A_Packfile ) return Boolean is

        function C_Pack_FError( f : A_Packfile ) return Integer;
        pragma Import( C, C_Pack_FError, "pack_ferror" );

    begin
        return C_Pack_FError( f ) /= 0;
    end Pack_FError;

    ----------------------------------------------------------------------------

    function Pack_FOpen( filename, mode : String ) return A_Packfile is

        function C_Pack_FOpen( filename, mode : C.char_array ) return A_Packfile;
        pragma Import( C, C_Pack_FOpen, "pack_fopen" );

    begin
        return C_Pack_FOpen( To_C( filename ), To_C( mode ) );
    end Pack_FOpen;

    ----------------------------------------------------------------------------

    function Pack_FOpen_Chunk( f : not null A_Packfile; pack : Boolean ) return A_Packfile is

        function C_Pack_FOpen_Chunk( f : A_Packfile; pack : Integer ) return A_Packfile;
        pragma Import( C, C_Pack_FOpen_Chunk, "pack_fopen_chunk" );

    begin
        if pack then
            return C_Pack_FOpen_Chunk( f, 1 );
        else
            return C_Pack_FOpen_Chunk( f, 0 );
        end if;
    end Pack_FOpen_Chunk;

    ----------------------------------------------------------------------------

    function Pack_FSeek( f : not null A_Packfile; offset : Integer ) return Boolean is

        function C_Pack_FSeek( f : A_Packfile; offset : Integer ) return Integer;
        pragma Import( C, C_Pack_FSeek, "pack_fseek" );

    begin
        return C_Pack_FSeek( f, offset ) = 0;
    end Pack_FSeek;

    ----------------------------------------------------------------------------

    procedure Packfile_Password( password : String ) is

        procedure C_Packfile_Password( password : C.char_array );
        pragma Import( C, C_Packfile_Password, "packfile_password" );

    begin
        C_Packfile_Password( To_C( password ) );
    end Packfile_Password;

    ----------------------------------------------------------------------------

    procedure Set_Allegro_Resource_Path( priority : Integer; path : String ) is

        function C_Set_Allegro_Resource_Path( priority : Integer;
                                              path     : C.char_array ) return Integer;
        pragma Import( C, C_Set_Allegro_Resource_Path, "set_allegro_resource_path" );

        tmp : Integer;
        pragma Warnings( off, tmp );
    begin
        tmp := C_Set_Allegro_Resource_Path( priority, To_C( path ) );
    end Set_Allegro_Resource_Path;

end Allegro.Files;
