
with Interfaces;                        use Interfaces;
with Interfaces.C;                      use Interfaces.C;
with Interfaces.C.Strings;              use Interfaces.C.Strings;

package body Alfont is

    function To_Ada_Boolean( val : Integer ) return Boolean is
    begin
        return val /= 0;
    end To_Ada_Boolean;
    pragma Inline_Always( To_Ada_Boolean );

    ----------------------------------------------------------------------------

    function To_C_Boolean( val : Boolean ) return Integer is
    begin
        if val then
            return 1;
        end if;
        return 0;
    end To_C_Boolean;
    pragma Inline_Always( To_C_Boolean );

    --==========================================================================

    function Alfont_Copyright return String is
    begin
        -- taken from README.txt in AlFont 2.0.9
        return "AllegroFont (c) 2001, 2002 Javier Gonzalez";
    end Alfont_Copyright;

    ----------------------------------------------------------------------------

    function Alfont_Version return String is
    begin
        return "AllegroFont 2.0.9";
    end Alfont_Version;

    ----------------------------------------------------------------------------

    function Load_Font( filepathname : String ) return A_Alfont_Font is

        function C_Load_Font( filepathname : C.char_array ) return A_Alfont_Font;
        pragma Import( C, C_Load_Font, "alfont_load_font" );

    begin
        return C_Load_Font( To_C( filepathname ) );
    end Load_Font;

    ----------------------------------------------------------------------------

    procedure Destroy_Font( f : in out A_Alfont_Font ) is

        procedure C_Destroy_Font( f : A_Alfont_Font );
        pragma Import( C, C_Destroy_Font, "alfont_destroy_font" );

    begin
        C_Destroy_Font( f );
        f := null;
    end Destroy_Font;

    ----------------------------------------------------------------------------

    function Is_Scalable_Font( f : not null A_Alfont_Font ) return Boolean is

        function C_Is_Scalable_Font( f : A_Alfont_Font ) return Integer;
        pragma Import( C, C_Is_Scalable_Font, "alfont_is_scalable_font" );

    begin
        return To_Ada_Boolean( C_Is_Scalable_Font( f ) );
    end Is_Scalable_Font;

    ----------------------------------------------------------------------------

    function Is_Fixed_Font( f : not null A_Alfont_Font ) return Boolean is

        function C_Is_Fixed_Font( f : A_Alfont_Font ) return Integer;
        pragma Import( C, C_Is_Fixed_Font, "alfont_is_fixed_font" );

    begin
        return To_Ada_Boolean( C_Is_Fixed_Font( f ) );
    end Is_Fixed_Font;

    ----------------------------------------------------------------------------

    procedure Set_Font_Fixed_Width( f : not null A_Alfont_Font; fixed_width : Boolean ) is

        procedure C_Set_Font_Fixed_Width( f : A_Alfont_Font; fixed_Width : Integer );
        pragma Import( C, C_Set_Font_Fixed_Width, "alfont_set_font_fixed_width" );

    begin
        C_Set_Font_Fixed_Width( f, To_C_Boolean( fixed_width ) );
    end Set_Font_Fixed_Width;

    ----------------------------------------------------------------------------

    function Get_Available_Fixed_Sizes( f : not null A_Alfont_Font ) return A_Size_Array is

        function C_Get_Available_Fixed_Sizes( f : A_Alfont_Font ) return Address;
        pragma Import( C, C_Get_Available_Fixed_Sizes, "alfont_get_available_fixed_sizes" );

        len : constant Integer := Get_Nof_Available_Fixed_Sizes( f );
    begin
        if len > 0 then
            declare
                fsa    : aliased Size_Array(1..len);
                for fsa'Address use C_Get_Available_Fixed_Sizes( f );

                result : constant A_Size_Array := new Size_Array'(fsa);
            begin
                for i in fsa'Range loop
                    result(i) := fsa(i);
                end loop;
                return result;
            end;
        else
            return null;
        end if;
    end Get_Available_Fixed_Sizes;

    ----------------------------------------------------------------------------

    procedure Set_Language( f : not null A_Alfont_Font; language : String ) is

        procedure C_Set_Language( f : A_Alfont_Font; language : C.char_array );
        pragma Import( C, C_Set_Language, "alfont_set_language" );

    begin
        C_Set_Language( f, To_C( language ) );
    end Set_Language;

    ----------------------------------------------------------------------------

    function Get_Language( f : not null A_Alfont_Font ) return String is

        function C_Get_Language( f : A_Alfont_Font ) return C.Strings.chars_ptr;
        pragma Import( C, C_Get_Language, "alfont_get_language" );

    begin
        return To_Ada( Value( C_Get_Language( f ) ) );
    end Get_Language;

    ----------------------------------------------------------------------------

    procedure Set_Font_Underline( f : not null A_Alfont_Font; underline : Boolean ) is

        procedure C_Set_Font_Underline( f : A_Alfont_Font; underline : Integer );
        pragma Import( C, C_Set_Font_Underline, "alfont_set_font_underline" );

    begin
        C_Set_Font_Underline( f, To_C_Boolean( underline ) );
    end Set_Font_Underline;

    ----------------------------------------------------------------------------

    function Get_Font_Underline( f : not null A_Alfont_Font ) return Boolean is

        function C_Get_Font_Underline( f : A_Alfont_Font ) return Integer;
        pragma Import( C, C_Get_Font_Underline, "alfont_get_font_underline" );

    begin
        return To_Ada_Boolean( C_Get_Font_Underline( f ) );
    end Get_Font_Underline;

    ----------------------------------------------------------------------------

    procedure Set_Font_Underline_Right( f : not null A_Alfont_Font; underline_right : Boolean ) is

        procedure C_Set_Font_Underline_Right( f : A_Alfont_Font; underline_right : Integer );
        pragma Import( C, C_Set_Font_Underline_Right, "alfont_set_font_underline_right" );

    begin
        C_Set_Font_Underline_Right( f, To_C_Boolean( underline_right ) );
    end Set_Font_Underline_Right;

    ----------------------------------------------------------------------------

    function Get_Font_Underline_Right( f : not null A_Alfont_Font ) return Boolean is

        function C_Get_Font_Underline_Right( f : A_Alfont_Font ) return Integer;
        pragma Import( C, C_Get_Font_Underline_Right, "alfont_get_font_underline_right" );

    begin
        return To_Ada_Boolean( C_Get_Font_Underline_Right( f ) );
    end Get_Font_Underline_Right;

    ----------------------------------------------------------------------------

    procedure Set_Font_Underline_Left( f : not null A_Alfont_Font; underline_left : Boolean ) is

        procedure C_Set_Font_Underline_Left( f : A_Alfont_Font; underline_left : Integer );
        pragma Import( C, C_Set_Font_Underline_Left, "alfont_set_font_underline_left" );

    begin
        C_Set_Font_Underline_Left( f, To_C_Boolean( underline_left ) );
    end Set_Font_Underline_Left;

    ----------------------------------------------------------------------------

    function Get_Font_Underline_Left( f : not null A_Alfont_Font ) return Boolean is

        function C_Get_Font_Underline_Left( f : A_Alfont_Font ) return Integer;
        pragma Import( C, C_Get_Font_Underline_Left, "alfont_get_font_underline_left" );

    begin
        return To_Ada_Boolean( C_Get_Font_Underline_Left( f ) );
    end Get_Font_Underline_Left;

    ----------------------------------------------------------------------------

    procedure Set_Font_Background( f : not null A_Alfont_Font; background : Boolean ) is

        procedure C_Set_Font_Background( f : A_Alfont_Font; background : Integer );
        pragma Import( C, C_Set_Font_Background, "alfont_set_font_background" );

    begin
        C_Set_Font_Background( f, To_C_Boolean( background ) );
    end Set_Font_Background;

    ----------------------------------------------------------------------------

    function Get_Font_Background( f : not null A_Alfont_Font ) return Boolean is

        function C_Get_Font_Background( f : A_Alfont_Font ) return Integer;
        pragma Import( C, C_Get_Font_Background, "alfont_get_font_background" );

    begin
        return To_Ada_Boolean( C_Get_Font_Background( f ) );
    end Get_Font_Background;

    ----------------------------------------------------------------------------

    procedure Set_Autofix( f : not null A_Alfont_Font; autofix : Boolean ) is

        procedure C_Set_Autofix( f : A_Alfont_Font; autofix : Integer );
        pragma Import( C, C_Set_Autofix, "alfont_set_autofix" );

    begin
        C_Set_Autofix( f, To_C_Boolean( autofix ) );
    end Set_Autofix;

    ----------------------------------------------------------------------------

    function Get_Autofix( f : not null A_Alfont_Font ) return Boolean is

        function C_Get_Autofix( f : A_Alfont_Font ) return Integer;
        pragma Import( C, C_Get_Autofix, "alfont_get_autofix" );

    begin
        return To_Ada_Boolean( C_Get_Autofix( f ) );
    end Get_Autofix;

    ----------------------------------------------------------------------------

    procedure Set_Font_Outline_Hollow( f : not null A_Alfont_Font; hollow : Boolean ) is

        procedure C_Set_Font_Outline_Hollow( f : A_Alfont_Font; hollow : Integer );
        pragma Import( C, C_Set_Font_Outline_Hollow, "alfont_set_font_outline_hollow" );

    begin
        C_Set_Font_Outline_Hollow( f, To_C_Boolean( hollow ) );
    end Set_Font_Outline_Hollow;

    ----------------------------------------------------------------------------

    function Get_Font_Outline_Hollow( f : not null A_Alfont_Font ) return Boolean is

        function C_Get_Font_Outline_Hollow( f : A_Alfont_Font ) return Integer;
        pragma Import( C, C_Get_Font_Outline_Hollow, "alfont_get_font_outline_hollow" );

    begin
        return To_Ada_Boolean( C_Get_Font_Outline_Hollow( f ) );
    end Get_Font_Outline_Hollow;

    ----------------------------------------------------------------------------

    function Text_Count( f : not null A_Alfont_Font; str : String ) return Natural is

        function C_Text_Count( f : A_Alfont_Font; str : C.char_array ) return Integer;
        pragma Import( C, C_Text_Count, "alfont_text_count" );

    begin
        return C_Text_Count( f, To_C( str ) );
    end Text_Count;

    ----------------------------------------------------------------------------

    function Need_Uconvert( f : not null A_Alfont_Font; str : String ) return Boolean is

        function C_Need_Uconvert( f : A_Alfont_Font; str : C.char_array ) return Integer;
        pragma Import( C, C_Need_Uconvert, "alfont_need_uconvert" );

    begin
        return To_Ada_Boolean( C_Need_Uconvert( f, To_C( str ) ) );
    end Need_Uconvert;

    ----------------------------------------------------------------------------

    procedure Textout_aa( bmp   : not null A_Bitmap;
                          f     : not null A_Alfont_Font;
                          s     : String;
                          x, y  : Integer;
                          color : Color_Type ) is

        procedure C_Textout_aa( bmp   : A_Bitmap;
                                f     : A_Alfont_Font;
                                s     : C.char_array;
                                x, y  : Integer;
                                color : Color_Type );
        pragma Import( C, C_Textout_aa, "alfont_textout_aa" );

    begin
        C_Textout_aa( bmp, f, To_C( s ), x, y, color );
    end Textout_aa;

    ----------------------------------------------------------------------------

    procedure Textout( bmp   : not null A_Bitmap;
                       f     : not null A_Alfont_Font;
                       s     : String;
                       x, y  : Integer;
                       color : Color_Type ) is

        procedure C_Textout( bmp   : A_Bitmap;
                             f     : A_Alfont_Font;
                             s     : C.char_array;
                             x, y  : Integer;
                             color : Color_Type );
        pragma Import( C, C_Textout, "alfont_textout" );

    begin
        C_Textout( bmp, f, To_C( s ), x, y, color );
    end Textout;

    ----------------------------------------------------------------------------

    procedure Textout_aa_ex( bmp   : not null A_Bitmap;
                             f     : not null A_Alfont_Font;
                             s     : String;
                             x, y  : Integer;
                             color : Color_Type;
                             backg : Integer ) is

        procedure C_Textout_aa_ex( bmp   : A_Bitmap;
                                   f     : A_Alfont_Font;
                                   s     : C.char_array;
                                   x, y  : Integer;
                                   color : Color_Type;
                                   backg : Integer );
        pragma Import( C, C_Textout_aa_ex, "alfont_textout_aa_ex" );

    begin
        C_Textout_aa_ex( bmp, f, To_C( s ), x, y, color, backg );
    end Textout_aa_ex;

    ----------------------------------------------------------------------------

    procedure Textout_ex( bmp   : not null A_Bitmap;
                          f     : not null A_Alfont_Font;
                          s     : String;
                          x, y  : Integer;
                          color : Color_Type;
                          backg : Integer ) is

        procedure C_Textout_ex( bmp   : A_Bitmap;
                                f     : A_Alfont_Font;
                                s     : C.char_array;
                                x, y  : Integer;
                                color : Color_Type;
                                backg : Integer );
        pragma Import( C, C_Textout_ex, "alfont_textout_ex" );

    begin
        C_Textout_ex( bmp, f, To_C( s ), x, y, color, backg );
    end Textout_ex;

    ----------------------------------------------------------------------------

    procedure Textout_Centre_aa( bmp   : not null A_Bitmap;
                                 f     : not null A_Alfont_Font;
                                 s     : String;
                                 x, y  : Integer;
                                 color : Color_Type ) is

        procedure C_Textout_Centre_aa( bmp   : A_Bitmap;
                                       f     : A_Alfont_Font;
                                       s     : C.char_array;
                                       x, y  : Integer;
                                       color : Color_Type );
        pragma Import( C, C_Textout_Centre_aa, "alfont_textout_centre_aa" );

    begin
        C_Textout_Centre_aa( bmp, f, To_C( s ), x, y, color );
    end Textout_Centre_aa;

    ----------------------------------------------------------------------------

    procedure Textout_Centre( bmp   : not null A_Bitmap;
                              f     : not null A_Alfont_Font;
                              s     : String;
                              x, y  : Integer;
                              color : Color_Type ) is

        procedure C_Textout_Centre( bmp   : A_Bitmap;
                                    f     : A_Alfont_Font;
                                    s     : C.char_array;
                                    x, y  : Integer;
                                    color : Color_Type );
        pragma Import( C, C_Textout_Centre, "alfont_textout_centre" );

    begin
        C_Textout_Centre( bmp, f, To_C( s ), x, y, color );
    end Textout_Centre;

    ----------------------------------------------------------------------------

    procedure Textout_Centre_aa_ex( bmp   : not null A_Bitmap;
                                    f     : not null A_Alfont_Font;
                                    s     : String;
                                    x, y  : Integer;
                                    color : Color_Type;
                                    backg : Integer ) is

        procedure C_Textout_Centre_aa_ex( bmp   : A_Bitmap;
                                          f     : A_Alfont_Font;
                                          s     : C.char_array;
                                          x, y  : Integer;
                                          color : Color_Type;
                                          backg : Integer );
        pragma Import( C, C_Textout_Centre_aa_ex, "alfont_textout_centre_aa_ex" );

    begin
        C_Textout_Centre_aa_ex( bmp, f, To_C( s ), x, y, color, backg );
    end Textout_Centre_aa_ex;

    ----------------------------------------------------------------------------

    procedure Textout_Centre_ex( bmp   : not null A_Bitmap;
                                 f     : not null A_Alfont_Font;
                                 s     : String;
                                 x, y  : Integer;
                                 color : Color_Type;
                                 backg : Integer ) is

        procedure C_Textout_Centre_ex( bmp   : A_Bitmap;
                                       f     : A_Alfont_Font;
                                       s     : C.char_array;
                                       x, y  : Integer;
                                       color : Color_Type;
                                       backg : Integer );
        pragma Import( C, C_Textout_Centre_ex, "alfont_textout_centre_ex" );

    begin
        C_Textout_Centre_ex( bmp, f, To_C( s ), x, y, color, backg );
    end Textout_Centre_ex;

    ----------------------------------------------------------------------------

    procedure Textout_Right_aa( bmp   : not null A_Bitmap;
                                f     : not null A_Alfont_Font;
                                s     : String;
                                x, y  : Integer;
                                color : Color_Type ) is

        procedure C_Textout_Right_aa( bmp   : A_Bitmap;
                                      f     : A_Alfont_Font;
                                      s     : C.char_array;
                                      x, y  : Integer;
                                      color : Color_Type );
        pragma Import( C, C_Textout_Right_aa, "alfont_textout_right_aa" );

    begin
        C_Textout_Right_aa( bmp, f, To_C( s ), x, y, color );
    end Textout_Right_aa;

    ----------------------------------------------------------------------------

    procedure Textout_Right( bmp   : not null A_Bitmap;
                             f     : not null A_Alfont_Font;
                             s     : String;
                             x, y  : Integer;
                             color : Color_Type ) is

        procedure C_Textout_Right( bmp   : A_Bitmap;
                                   f     : A_Alfont_Font;
                                   s     : C.char_array;
                                   x, y  : Integer;
                                   color : Color_Type );
        pragma Import( C, C_Textout_Right, "alfont_textout_right" );

    begin
        C_Textout_Right( bmp, f, To_C( s ), x, y, color );
    end Textout_Right;

    ----------------------------------------------------------------------------

    procedure Textout_Right_aa_ex( bmp   : not null A_Bitmap;
                                   f     : not null A_Alfont_Font;
                                   s     : String;
                                   x, y  : Integer;
                                   color : Color_Type;
                                   backg : Integer ) is

        procedure C_Textout_Right_aa_ex( bmp   : A_Bitmap;
                                         f     : A_Alfont_Font;
                                         s     : C.char_array;
                                         x, y  : Integer;
                                         color : Color_Type;
                                         backg : Integer );
        pragma Import( C, C_Textout_Right_aa_ex, "alfont_textout_right_aa_ex" );

    begin
        C_Textout_Right_aa_ex( bmp, f, To_C( s ), x, y, color, backg );
    end Textout_Right_aa_ex;

    ----------------------------------------------------------------------------

    procedure Textout_Right_ex( bmp   : not null A_Bitmap;
                                f     : not null A_Alfont_Font;
                                s     : String;
                                x, y  : Integer;
                                color : Color_Type;
                                backg : Integer ) is

        procedure C_Textout_Right_ex( bmp   : A_Bitmap;
                                      f     : A_Alfont_Font;
                                      s     : C.char_array;
                                      x, y  : Integer;
                                      color : Color_Type;
                                      backg : Integer );
        pragma Import( C, C_Textout_Right_ex, "alfont_textout_right_ex" );

    begin
        C_Textout_Right_ex( bmp, f, To_C( s ), x, y, color, backg );
    end Textout_Right_ex;

    ----------------------------------------------------------------------------

    function Text_Length( f : not null A_Alfont_Font; str : String ) return Natural is

        function C_Text_Length( f : A_Alfont_Font; str : C.char_array ) return Natural;
        pragma Import( C, C_Text_Length, "alfont_text_length" );

    begin
        return C_Text_Length( f, To_C( str ) );
    end Text_Length;

end Alfont;
