
--with Debugging;                         use Debugging;
with Entities.Players;                  use Entities.Players;
with Resources;                         use Resources;
with Tiles.Libraries;                   use Tiles.Libraries;
with Worlds;                            use Worlds;

package body Entities.Triggers is

    DEFAULT_WIDTH  : constant := 32;
    DEFAULT_HEIGHT : constant := 32;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this    : access Trigger;
                         libName : String;
                         icon    : String ) is
    begin
        Entity(this.all).Construct( DEFAULT_WIDTH, DEFAULT_HEIGHT, libName );
        this.frame := this.lib.Get_Id( icon );
        if this.frame = 0 then
            raise RESOURCE_ERROR with "Image <" & icon & "> not found in " &
                                      "tile library <" & libName & ">";
        end if;
        this.metaphysical := True;
        this.physical := False;
        this.clipped := False;
        this.attrs.Set_Value( ACTION_ATTR, "" );
        this.attrs.Set_Value( ARG_ATTR, "" );
        this.attrs.Set_Value( CONDITION_ATTR, "" );
        this.attrs.Set_Value( WHEN_ATTR, "OnActivate" );
    end Construct;

    ----------------------------------------------------------------------------

    function Evaluate( this       : not null access Trigger'Class;
                       expression : String ) return Boolean is
        result : Boolean := False;
        value  : A_Value;
    begin
        value := this.world.Evaluate( expression );
        result := value /= null and then value.As_Boolean;
        Delete( value );
        return result;
    end Evaluate;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Trigger ) is
    begin
        Entity'Read( stream, Entity(obj) );
        obj.activatable := Boolean'Input( stream );
        obj.attrs.Set_Default( ACTION_ATTR, "" );
        obj.attrs.Set_Default( ARG_ATTR, "" );
        obj.attrs.Set_Default( CONDITION_ATTR, "" );
        obj.attrs.Set_Default( WHEN_ATTR, "" );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Trigger ) is
    begin
        Entity'Write( stream, Entity(obj) );
        Boolean'Output( stream, obj.activatable );
    end Object_Write;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Activate( this      : access Trigger;
                           activator : not null A_Entity ) is
    begin
        if this.activatable and then
           not this.editingMode and then
           activator.all in Player'Class and then
           this.attrs.As_String( WHEN_ATTR ) = "OnActivate" and then
           (this.attrs.As_String( CONDITION_ATTR )'Length = 0 or else
            this.Evaluate( this.attrs.As_String( CONDITION_ATTR ) ))
        then
            A_Trigger(this).Execute( this.attrs.As_String( ACTION_ATTR ),
                                     this.attrs.As_String( ARG_ATTR ),
                                     activator );
        end if;
    end On_Activate;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Collide( this : access Trigger; e : not null A_Entity ) is
    begin
        if this.activatable and then
           not this.editingMode and then
           e.all in Player'Class and then
           this.attrs.As_String( WHEN_ATTR ) = "OnTouch" and then
           (this.attrs.As_String( CONDITION_ATTR )'Length = 0 or else
            this.Evaluate( this.attrs.As_String( CONDITION_ATTR ) ))
        then
            A_Trigger(this).Execute( this.attrs.As_String( ACTION_ATTR ),
                                     this.attrs.As_String( ARG_ATTR ),
                                     e );
        end if;
    end On_Collide;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Load( this : access Trigger ) is
    begin
        Entity(this.all).On_Load;

        -- reset the activatable flag so that a trigger can be activated again
        -- if it was activated once and then the game session was saved or the
        -- world was stored to memory and later resumed.
        this.activatable := True;

        if this.activatable and then
           not this.editingMode and then
           this.attrs.As_String( WHEN_ATTR ) = "OnLoad" and then
           (this.attrs.As_String( CONDITION_ATTR )'Length = 0 or else
            this.Evaluate( this.attrs.As_String( CONDITION_ATTR ) ))
        then
            A_Trigger(this).Execute( this.attrs.As_String( ACTION_ATTR ),
                                     this.attrs.As_String( ARG_ATTR ),
                                     null );
        end if;
    end On_Load;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Trigger ) is
    begin
        Delete( A_Object(this) );
    end Delete;

end Entities.Triggers;
