
with Ada.Streams.Stream_IO;             use Ada.Streams.Stream_IO;
with Streams.Buffers;                   use Streams.Buffers;

package body Expressions.Evaluators is

    function Create_Evaluator return A_Evaluator is
        this : constant A_Evaluator := new Evaluator;
    begin
        this.Construct;
        return this;
    end Create_Evaluator;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Evaluator ) is
    begin
        Delete( this.parser );
        Delete( this.scanner );
        Limited_Object(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    procedure Evaluate( this       : not null access Evaluator'Class;
                        expression : String;
                        resolver   : A_Symbol_Resolver;
                        result     : in out A_Value ) is
        input   : A_Buffer_Stream := Stream( expression );
        exp     : A_Expression;
        context : A_Eval_Context;
    begin
        Delete( result );
        this.scanner.Set_Input( Stream_Access(input) );
        exp := this.parser.Scan_Expression( this.scanner );
        this.scanner.Expect( TK_EOF );    -- may raise exception
        if exp /= null then
            context := Create_Eval_Context( resolver );
            result := exp.Evaluate( context );
            -- evaluate expression
            Delete( exp );
        end if;
        this.scanner.Set_Input( null );
        Close( input );
    exception
        when others =>
            Delete( exp );
            this.scanner.Set_Input( null );
            Close( input );
            raise;
    end Evaluate;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Evaluator ) is
    begin
        Delete( A_Limited_Object(this) );
    end Delete;

end Expressions.Evaluators;
