
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Font_API.Alfont_Fonts;
with Font_API.Allegro_Fonts;
with Resources;                         use Resources;

package body Font_API is

    initialized,
    finalized   : Boolean := False;

    ----------------------------------------------------------------------------

    procedure Initialize is
    begin
        if not initialized then
            pragma Debug( Dbg( "Initializing font system", D_INIT, INFO ) );
            Alfont_Fonts.Initialize;
            initialized := True;
        end if;
    end Initialize;

    ----------------------------------------------------------------------------

    procedure Finalize is
    begin
        if initialized and then not finalized then
            pragma Debug( Dbg( "Finalizing font system", D_INIT, INFO ) );
            Alfont_Fonts.Finalize;
            finalized := True;
        end if;
    end Finalize;

    --==========================================================================

    function Load_Font( filename : String; size : Positive ) return Font_Type is
        this : Font_Type;
    begin
        this := Alfont_Fonts.Load_Font( filename, size );
        if this = null then
            this := Allegro_Fonts.Load_Font( filename, size );
        end if;

        if this = null then
            raise RESOURCE_ERROR with "Error loading font <" & filename & ">";
        end if;
        return this;
    end Load_Font;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out Font_Type ) is
    begin
        Delete( A_Limited_Object(this) );
    end Delete;

end Font_API;
