
with Allegro.Drawing;                   use Allegro.Drawing;
with Allegro.Graphics;                  use Allegro.Graphics;
with Allegro.Mouse;                     use Allegro.Mouse;
with Allegro.Sprites;                   use Allegro.Sprites;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Widgets;                           use Widgets;

package body Renderers is

    function Create_Renderer( win : not null A_Window;
                              fps : Positive ) return A_Renderer is
        this : constant A_Renderer := new Renderer;
    begin
        this.Construct( win, fps );
        return this;
    end Create_Renderer;

    ----------------------------------------------------------------------------

    overriding
    procedure Adjust( this : access Renderer ) is
    begin
        Object(this.all).Adjust;
        for i in this.videoPages'Range loop
            this.videoPages(i) := Create_Video_Bitmap(
                                      Get_Width( this.videoPages(i) ),
                                      Get_Height( this.videoPages(i) ) );
            Clear_To_Color( this.videoPages(i), 0 );
        end loop;
    end Adjust;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Renderer;
                         win  : not null A_Window;
                         fps  : Positive ) is
        sw : constant Integer := Get_Screen_W;
        sh : constant Integer := Get_Screen_H;
    begin
        Object(this.all).Construct;
        this.frameDelta := Milliseconds( 1000 / fps );
        this.lastFrame := Clock - this.frameDelta;
        this.win := win;
        this.win.Set_Offset( sw / 2 - win.Get_Width * win.Get_Scale / 2,
                             sh / 2 - win.Get_Height * win.Get_Scale / 2 );
        for i in this.videoPages'Range loop
            this.videoPages(i) := Create_Video_Bitmap( sw, sh );
            Clear_To_Color( this.videoPages(i), 0 );
        end loop;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Renderer ) is
    begin
        if using_sw_mouse then
            Show_Mouse( null );
        end if;
        for i in this.videoPages'Range loop
            Destroy_Bitmap( this.videoPages(i) );
        end loop;
        Object(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    procedure Draw_Frame( this  : not null access Renderer'Class;
                          vpage : not null A_Bitmap;
                          dt    : Time_Span ) is
        pragma Unreferenced( dt );
    begin
        this.win.Draw( vpage );
    end Draw_Frame;

    ----------------------------------------------------------------------------

    overriding
    function Get_Process_Name( this : access Renderer ) return String is
        pragma Unreferenced( this );
    begin
        return "Renderer";
    end Get_Process_Name;

    ----------------------------------------------------------------------------

    overriding
    procedure Tick( this : access Renderer; time : Tick_Time ) is
        now : constant Ada.Real_Time.Time := Clock;
    begin
        if now - this.frameDelta > this.lastFrame then
            Acquire_Bitmap( this.videoPages(this.drawPage) );
            begin
                this.Draw_Frame( this.videoPages(this.drawPage), time.elapsed );
                if using_sw_mouse then
                    Draw_Sprite( this.videoPages(this.drawPage), Get_Mouse_Sprite, Mouse_X, Mouse_Y );
                end if;
                Release_Bitmap( this.videoPages(this.drawPage) );
            exception
                when others =>
                    Release_Bitmap( this.videoPages(this.drawPage) );
                    -- this will be caught by the Process_Manager and terminate
                    -- the application.
                    raise;
            end;
            if Show_Video_Bitmap( this.videoPages(this.drawPage) ) /= 0 then
                null;
                pragma Debug( Dbg( "Renderer Error: " &
                                   "Unable to display video bitmap",
                                   D_GUI, ERROR ) );
            end if;
            this.drawPage := this.drawPage + 1;
            this.lastFrame := now;
        end if;
    end Tick;

    --==========================================================================

    function Copy( src : A_Renderer ) return A_Renderer is
    begin
        return A_Renderer(Copy( A_Object(src) ));
    end Copy;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Renderer ) is
    begin
        Delete( A_Object(this) );
    end Delete;

    ----------------------------------------------------------------------------

    procedure Use_Software_Mouse is
    begin
        using_sw_mouse := True;
    end Use_Software_Mouse;

end Renderers;
