
with Ada.Unchecked_Deallocation;

package body Streams.Packfiles is

    procedure Free is new Ada.Unchecked_Deallocation( Packfile_Stream'Class,
                                                      A_Packfile_Stream );

    ----------------------------------------------------------------------------

    function Open_Stream( filename, mode : String ) return A_Packfile_Stream is
        strm : A_Packfile_Stream := new Packfile_Stream;
    begin
        strm.packfile := Pack_FOpen( filename, mode );
        if strm.packfile /= null then
            strm.doClose := True;
        else
            Free( strm );
        end if;
        return strm;
    end Open_Stream;

    ----------------------------------------------------------------------------

    function Stream( f : not null A_Packfile ) return A_Packfile_Stream is
        strm : constant A_Packfile_Stream := new Packfile_Stream;
    begin
        strm.packfile := f;
        return strm;
    end Stream;

    ----------------------------------------------------------------------------

    function EOF( stream : not null access Packfile_Stream'Class ) return Boolean is
    begin
        return Pack_FEOF( stream.packfile );
    end EOF;

    ----------------------------------------------------------------------------

    function Error( stream : not null access Packfile_Stream'Class ) return Boolean is
    begin
        return Pack_FError( stream.packfile );
    end Error;

    ----------------------------------------------------------------------------

    overriding
    procedure Read( stream : in out Packfile_Stream;
                    item   : out Stream_Element_Array;
                    last   : out Stream_Element_Offset ) is
    begin
        last := Stream_Element_Offset(
            Pack_FRead( item(item'First)'Address, item'Length, stream.packfile ));
    end Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Write( stream : in out Packfile_Stream;
                     item   : Stream_Element_Array ) is
        ret : Integer;
        pragma Warnings( Off, ret );
    begin
        ret := Pack_FWrite( item(item'First)'Address, item'Length, stream.packfile );
    end Write;

    ----------------------------------------------------------------------------

    procedure Close( stream : in out A_Packfile_Stream ) is
    begin
        if stream /= null then
            if stream.doClose then
                Pack_FClose0( stream.packfile );
            end if;
            Free( stream );
        end if;
    end Close;

end Streams.Packfiles;
