
with Expressions.Operators;             use Expressions.Operators;

package body Tokens.Scanners is

    function Create_Token_Scanner return A_Token_Scanner is
        this : constant A_Token_Scanner := new Token_Scanner;
    begin
        this.Construct;
        return this;
    end Create_Token_Scanner;

    ----------------------------------------------------------------------------

    function Accept_Binary_Op( this : not null access Token_Scanner'Class ) return A_Token is
        token : A_Token;
    begin
        token := this.Get_Next;
        if not Expressions.Operators.Is_Binary( token.Get_Type ) then
            this.Push_Back( token );
        end if;
        return token;
    end Accept_Binary_Op;

    ----------------------------------------------------------------------------

    function Accept_Identifier( this : not null access Token_Scanner'Class ) return A_Identifier_Token is
    begin
        return A_Identifier_Token(this.Accept_Token( TK_IDENTIFIER ));
    end Accept_Identifier;
    pragma Inline_Always( Accept_Identifier );

    ----------------------------------------------------------------------------

    function Accept_Token( this      : not null access Token_Scanner'Class;
                           tokenType : Token_Type ) return A_Token is
        token : A_Token;
    begin
        token := this.Get_Next;
        if token.Get_Type /= tokenType then
            this.Push_Back( token );
        end if;
        return token;
    end Accept_Token;

    ----------------------------------------------------------------------------

    function Accept_Unary_Op( this : not null access Token_Scanner'Class ) return A_Token is
        token : A_Token;
    begin
        token := this.Get_Next;
        if not Expressions.Operators.Is_Unary( token.Get_Type ) then
            this.Push_Back( token );
        end if;
        return token;
    end Accept_Unary_Op;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Token_Scanner ) is
    begin
        Delete( this.tokenizer );
        Limited_Object(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    function Expect_Identifier( this : not null access Token_Scanner'Class ) return A_Identifier_Token is
    begin
        return A_Identifier_Token(this.Expect_Token( TK_IDENTIFIER ));
    end Expect_Identifier;
    pragma Inline_Always( Expect_Identifier );

    ----------------------------------------------------------------------------

    function Expect_Token( this      : not null access Token_Scanner'Class;
                           tokenType : Token_Type ) return A_Token is
        token     : A_Token;
        foundType : Token_Type;
        loc       : Token_Location;
    begin
        token := this.Get_Next;
        foundType := token.Get_Type;
        if foundType /= tokenType then
            loc := token.Get_Location;
            this.Push_Back( token );
            raise Parse_Exception with "Expected " & tokenType'Img &
                ", found " & foundType'Img & " at " & To_String( loc );
        end if;
        return token;
    end Expect_Token;

    ----------------------------------------------------------------------------

    procedure Expect( this      : not null access Token_Scanner'Class;
                            tokenType : Token_Type ) is
        token : A_Token;
    begin
        token := this.Expect_Token( tokenType );
        Delete( token );
    end Expect;

    ----------------------------------------------------------------------------

    function Get_Location( this : not null access Token_Scanner'Class ) return Token_Location is
    begin
        return this.tokenizer.Get_Location;
    end Get_Location;

    ----------------------------------------------------------------------------

    function Get_Next( this : not null access Token_Scanner'Class ) return A_Token is
    begin
        return this.tokenizer.Get_Next;
    end Get_Next;
    pragma Inline_Always( Get_Next );

    ----------------------------------------------------------------------------

    procedure Push_Back( this  : not null access Token_Scanner'Class;
                         token : in out A_Token ) is
    begin
        this.tokenizer.Push_Back( token );
    end Push_Back;
    pragma Inline_Always( Push_Back );

    ----------------------------------------------------------------------------

    procedure Set_Input( this   : not null access Token_Scanner'Class;
                         stream : Stream_Access ) is
    begin
        this.tokenizer.Set_Input( stream );
    end Set_Input;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Token_Scanner ) is
    begin
        Delete( A_Limited_Object(this) );
    end Delete;

end Tokens.Scanners;
