
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Mouse;                             use Mouse;

package body Widgets.Buttons.Pushes is

    function Create_Push_Button( view : not null access Game_Views.Game_View'Class;
                                 id   : String;
                                 text : String := "";
                                 icon : String := "" ) return A_Button is
        this : A_Push_Button := new Push_Button;
    begin
        this.Construct( view, id, text, icon );
        return A_Button(this);
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Push_Button;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Blur( this : access Push_Button ) is
    begin
        Button(this.all).Handle_Blur;
        this.Set_State( False );
    end Handle_Blur;

    ----------------------------------------------------------------------------

    overriding
    function Handle_Key_Press( this : access Push_Button;
                               evt  : not null A_Key_Event ) return Boolean is
    begin
        if evt.Get_Key = KEY_SPACE or else
           evt.Get_Key = KEY_ENTER or else
           evt.Get_Key = KEY_ENTER_PAD
        then
            this.Set_State( True );
            return True;
        end if;
        return Button(this.all).Handle_Key_Press( evt );
    end Handle_Key_Press;

    ----------------------------------------------------------------------------

    overriding
    function Handle_Key_Release( this : access Push_Button;
                                 evt  : not null A_Key_Event ) return Boolean is
    begin
        if evt.Get_Key = KEY_SPACE or else
           evt.Get_Key = KEY_ENTER or else
           evt.Get_Key = KEY_ENTER_PAD
        then
            this.Set_State( False );
            this.Dispatch_Action( Click );
            return True;
        end if;
        return Button(this.all).Handle_Key_Release( evt );
    end Handle_Key_Release;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Mouse_Held( this : access Push_Button;
                                 evt  : not null A_Mouse_Button_Event ) is
    begin
        if evt.Get_Button = Mouse_Left then
            this.Dispatch_Action( Held );
        end if;
    end Handle_Mouse_Held;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Mouse_Press( this : access Push_Button;
                                  evt  : not null A_Mouse_Button_Event ) is
    begin
        if evt.Get_Button = Mouse_Left then
            this.Set_State( True );
        end if;
    end Handle_Mouse_Press;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Mouse_Release( this : access Push_Button;
                                    evt  : not null A_Mouse_Button_Event ) is
        x : constant Integer := evt.Get_X;
        y : constant Integer := evt.Get_Y;
    begin
        if evt.Get_Button = Mouse_Left then
            this.Set_State( False );
            if x >= 0 and then y >= 0 and then
               x < this.Get_Width and then y < this.Get_Height
            then
                this.Dispatch_Action( Click );
            end if;
        end if;
    end Handle_Mouse_Release;

    ----------------------------------------------------------------------------

    overriding
    procedure Toggle_State( this : access Push_Button ) is
    begin
        -- Change the state twice because it's a push button; it's not supposed
        -- to stay pressed down.
        A_Button(this).Set_State( not this.on );
        A_Button(this).Set_State( not this.on );
    end Toggle_State;

end Widgets.Buttons.Pushes;
