
with Widgets.Menu_Items;                use Widgets.Menu_Items;

package Widgets.Containers.Pulldown_Menus is

    -- A Pulldown_Menu is a container for menu items in the window's menu bar.
    -- It automatically sizes itself to fit the Menu_Items it contains.
    type Pulldown_Menu is new Container and Menu_Listener with private;
    type A_Pulldown_Menu is access all Pulldown_Menu'Class;

    -- Creates a new pulldown menu within 'view' with id 'id. 'text' is the
    -- name of the pulldown menu to be displayed in a parent menu. (ie: in the
    -- menu bar.)
    function Create_Pulldown_Menu( view : not null access Game_Views.Game_View'Class;
                                   id   : String;
                                   text : String ) return A_Pulldown_Menu;
    pragma Precondition( id'Length > 0 );
    pragma Postcondition( Create_Pulldown_Menu'Result /= null );

    -- Adds 'child' to the bottom of the pulldown menu. Only Menu_Item widgets
    -- should be added, otherwise it will be added but useless. 'child' will be
    -- consumed if 'consume' is True.
    procedure Add( this    : access Pulldown_Menu;
                   child   : in out A_Widget;
                   consume : Boolean := True );
    pragma Precondition( child /= null );
    pragma Postcondition( consume xor child /= null );

    -- Returns the pulldown menu's text to be displayed on the menu bar.
    function Get_Text( this : access Pulldown_Menu ) return String;

private

    type Pulldown_Menu is new Container and Menu_Listener with
        record
            text : Unbounded_String;
        end record;

    -- Constructs the pulldown menu.
    procedure Construct( this    : access Pulldown_Menu;
                         view    : not null access Game_Views.Game_View'Class;
                         id      : String;
                         text    : String );
    pragma Precondition( id'Length > 0 );

    -- Draws the background of the pulldown menu.
    procedure Draw_Content( this : access Pulldown_Menu; dc : Drawing_Context );

    -- Returns the minimum height of the pulldown menu, based on the combined
    -- heights of the children.
    function Get_Min_Height( this : access Pulldown_Menu ) return Natural;

    -- Returns the minimum width of the pulldown menu, based on the width of the
    -- widest child.
    function Get_Min_Width( this : access Pulldown_Menu ) return Natural;

    -- Handles Menu_Actions, hiding the pulldown menu when a child is clicked.
    procedure Handle_Action( this   : access Pulldown_Menu;
                             action : A_Menu_Action );

    -- Packs the children and then the pulldown menu.
    procedure Pack( this : access Pulldown_Menu );

    -- Removes 'child' from the pulldown menu container.
    procedure Remove( this : access Pulldown_Menu; child : not null A_Widget );

    -- Returns a string representation of the pulldown menu. Useful for
    -- debugging purposes.
    function To_String( this : access Pulldown_Menu ) return String;

end Widgets.Containers.Pulldown_Menus;
