
package Tiles.Libraries.Packing is

    type A_Output_Proc is
        access procedure( msg : String );

    function Pack_Library( libPath    : String;    -- path of output library
                           indexPath  : String;    -- path of input tile index
                           matrixPath : String;    -- path of input matrix index
                           compress   : Boolean;
                           Output     : A_Output_Proc
                         ) return Boolean;
    pragma Precondition( indexPath'Length > 0 );
    pragma Precondition( libPath'Length > 0 );

private

    -- Step 1: returns a tile library if manifest was successfully parsed
    function Parse_Manifest( libPath    : String;
                             indexPath  : String;
                             matrixPath : String;
                             Put_Line   : not null A_Output_Proc
                           ) return A_Tile_Library;
    pragma Precondition( indexPath'Length > 0 );
    pragma Precondition( libPath'Length > 0 );

    -- Step 2: returns true if all referenced files exist
    -- 'indexPath' is the path of the text index file used to create the library
    function Check_Dependencies( this      : not null A_Tile_Library;
                                 indexPath : String;
                                 Put_Line  : not null A_Output_Proc
                               ) return Boolean;

    -- Step 3: returns true if index file successfully created
    function Write_Index( this     : not null A_Tile_Library;
                          Put_Line : not null A_Output_Proc ) return Boolean;

    -- Step 4: returns true if library archive was successfully created
    function Create_Archive( this     : not null A_Tile_Library;
                             libPath  : String;
                             compress : Boolean;
                             Put_Line : not null A_Output_Proc
                           ) return Boolean;

end Tiles.Libraries.Packing;
