
--with Debugging;                         use Debugging;
with Events.Manager;                    use Events.Manager;

package body Events.Ked is

    not overriding
    procedure Construct( this      : access Apply_Tool_Map_Event;
                         func      : Function_Type;
                         modifiers : Modifiers_Array;
                         first     : Boolean;
                         x, y      : Integer;
                         layer     : Integer ) is
    begin
        Event(this.all).Construct( "Apply_Tool_Map" );
        this.func := func;
        this.modifiers := modifiers;
        this.first := first;
        this.x := x;
        this.y := y;
        this.layer := layer;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Function( this : not null access Apply_Tool_Map_Event'Class ) return Function_Type is
    begin
        return this.func;
    end Get_Function;

    ----------------------------------------------------------------------------

    function Get_Layer( this : not null access Apply_Tool_Map_Event'Class ) return Integer is
    begin
        return this.layer;
    end Get_Layer;

    ----------------------------------------------------------------------------

    function Get_Modifiers( this : not null access Apply_Tool_Map_Event'Class ) return Modifiers_Array is
    begin
        return this.modifiers;
    end Get_Modifiers;

    ----------------------------------------------------------------------------

    function Get_X( this : not null access Apply_Tool_Map_Event'Class ) return Integer is
    begin
        return this.x;
    end Get_X;

    ----------------------------------------------------------------------------

    function Get_Y( this : not null access Apply_Tool_Map_Event'Class ) return Integer is
    begin
        return this.y;
    end Get_Y;

    ----------------------------------------------------------------------------

    function Is_First( this : not null access Apply_Tool_Map_Event'Class ) return Boolean is
    begin
        return this.first;
    end Is_First;

    --==========================================================================

    procedure Construct( this      : access Import_World_Event;
                         filename  : String;
                         libName   : String;
                         domain    : String;
                         tolerance : Natural ) is
    begin
        Event(this.all).Construct( "Import_World" );
        this.filename := To_Unbounded_String( filename );
        this.libName := To_Unbounded_String( libName );
        this.domain := To_Unbounded_String( domain );
        this.tolerance := tolerance;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Domain( this : not null access Import_World_Event'Class ) return String is
    begin
        return To_String( this.domain );
    end Get_Domain;

    ----------------------------------------------------------------------------

    function Get_Filename( this : not null access Import_World_Event'Class ) return String is
    begin
        return To_String( this.filename );
    end Get_Filename;

    ----------------------------------------------------------------------------

    function Get_Library_Name( this : not null access Import_World_Event'Class ) return String is
    begin
        return To_String( this.libName );
    end Get_Library_Name;

    ----------------------------------------------------------------------------

    function Get_Tolerance( this : not null access Import_World_Event'Class ) return Natural is
    begin
        return this.tolerance;
    end Get_Tolerance;

    --==========================================================================

    overriding
    procedure Construct( this : access Save_World_Event; filename : String ) is
    begin
        Event(this.all).Construct( "Save_World" );
        this.filename := To_Unbounded_String( filename );
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Filename( this : not null access Save_World_Event'Class ) return String is
    begin
        return To_String( this.filename );
    end Get_Filename;

    --==========================================================================

    overriding
    procedure Adjust( this : access Set_Tool_Event ) is
    begin
        Event(this.all).Adjust;
        this.tool := Copy( this.tool );
    end Adjust;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Set_Tool_Event; tool : not null A_Tool ) is
    begin
        Event(this.all).Construct( "Set_Tool" );
        this.tool := Copy( tool );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Set_Tool_Event ) is
    begin
        Delete( this.tool );
        Event(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    function Get_Tool( this : not null access Set_Tool_Event'Class ) return A_Tool is
    begin
        return this.tool;
    end Get_Tool;

    --==========================================================================

    procedure Queue_Apply_Tool_Map( func      : Function_Type;
                                    modifiers : Modifiers_Array;
                                    first     : Boolean;
                                    x, y      : Integer;
                                    layer     : Integer ) is
        evt : A_Apply_Tool_Map_Event := new Apply_Tool_Map_Event;
    begin
        evt.Construct( func, modifiers, first, x, y, layer );
        Queue_Event( A_Event(evt) );
    end Queue_Apply_Tool_Map;

    ----------------------------------------------------------------------------

    procedure Trigger_Import_World( filename  : String;
                                    libName   : String;
                                    domain    : String;
                                    tolerance : Natural ) is
        evt  : A_Import_World_Event := new Import_World_Event;
        resp : Response_Type;
    begin
        evt.Construct( filename, libName, domain, tolerance );
        Trigger_Event( A_Event(evt), resp );
        if Get_Status( resp ) /= ST_SUCCESS then
            raise Program_Error with Get_Message( resp );
        end if;
    end Trigger_Import_World;

    ----------------------------------------------------------------------------

    procedure Trigger_Save_World( filename : String ) is
        evt  : A_Save_World_Event := new Save_World_Event;
        resp : Response_Type;
    begin
        evt.Construct( filename );
        Trigger_Event( A_Event(evt), resp );
        if Get_Status( resp ) /= ST_SUCCESS then
            raise Program_Error with Get_Message( resp );
        end if;
    end Trigger_Save_World;

    ----------------------------------------------------------------------------

    procedure Queue_Set_Tool( tool : not null A_Tool ) is
        evt : A_Set_Tool_Event := new Set_Tool_Event;
    begin
        evt.Construct( tool );
        Queue_Event( A_Event(evt) );
    end Queue_Set_Tool;

end Events.Ked;
