
with Ada.Exceptions;                    use Ada.Exceptions;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Themes;                            use Themes;
with Scaling;                           use Scaling;

package body Game_Views.Ked.Actions is

    procedure Action_Import( view : not null A_Ked_View ) is
    begin
        view.Do_Dialog_Action( DA_Import, saveChanges => True );
    end Action_Import;

    ----------------------------------------------------------------------------

    procedure Action_New( view : not null A_Ked_View ) is
    begin
        view.Do_Dialog_Action( DA_New, saveChanges => True );
    end Action_New;

    ----------------------------------------------------------------------------

    procedure Action_Open( view : not null A_Ked_View ) is
    begin
        view.Do_Dialog_Action( DA_Open, saveChanges => True );
    end Action_Open;

    ----------------------------------------------------------------------------

    procedure Action_Quit( view : not null A_Ked_View ) is
    begin
        view.Do_Dialog_Action( DA_Quit, saveChanges => True );
    end Action_Quit;

    ----------------------------------------------------------------------------

    procedure Action_Saveas( view : not null A_Ked_View ) is
    begin
        view.Do_Dialog_Action( DA_Save, saveChanges => False );
    end Action_Saveas;

    ----------------------------------------------------------------------------

    procedure Action_Save( view : not null A_Ked_View ) is
        filename : constant String := view.Get_Filename;
    begin
        if filename'Length > 0 then
            begin
                view.Save_World( filename );
            exception
                when e : others =>
                    null;
                    pragma Debug( Dbg( "Error saving file: " &
                                       Exception_Message( e ), D_VIEW, ERROR ) );
            end;
        else
            Action_Saveas( view );
        end if;
    end Action_Save;

    ----------------------------------------------------------------------------

    procedure Action_Toggle_Filter( view : not null A_Ked_View ) is
        theme : constant A_Theme := Themes.Get;
        pragma Unreferenced( view );
    begin
        theme.Set_Scaling_Filter( Filter_Type'Succ( theme.Get_Scaling_Filter ) );
    exception
        when Constraint_Error =>
            -- wrapped around end of filter list
            theme.Set_Scaling_Filter( Filter_Type'First );
    end Action_Toggle_Filter;

end Game_Views.Ked.Actions;
