
--with Debugging;                         use Debugging;
with Entities;                          use Entities;
with Events.Ked;                        use Events.Ked;
with Preferences;                       use Preferences;
with Support;
with Tools.Pointers;                    use Tools.Pointers;

package body Tools.Entity_Spawners is

    function Create_Entity_Spawner( id : String; snap : Boolean := True ) return A_Tool is
        this : constant A_Entity_Spawner := new Entity_Spawner;
    begin
        this.Construct( id, snap );
        return A_Tool(this);
    end Create_Entity_Spawner;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this      : access Entity_Spawner;
                     func      : Function_Type;
                     modifiers : Modifiers_Array;
                     first     : Boolean;
                     world     : not null A_World;
                     worldX,
                     worldY,
                     layer     : Integer ) is
        pragma Unreferenced( layer );

        ------------------------------------------------------------------------

        function Snap( n : Integer; size : Natural ) return Float is
            gridSize : constant Integer := Get_Pref( "gridsize" );
        begin
            if this.snap and then (Get_Pref( "gridsnap" ) xor modifiers(CTRL)) then
                -- Snap the entity if it's snappable (this.snap) and gridsnap is
                -- on. Allow the Ctrl key to invert the snapping behavior.
                -- Snap the entity to the center of the grid unless it's an even
                -- number of tiles wide. (For example, a 2x2 tile entity should
                -- be placed exactly over 2x2 tiles, which requires snapping to
                -- the edges of the grid and not the center.)
                return Float(Support.Grid_Snap( n,
                                                gridSize,
                                                centered => size mod (gridSize * 2) /= 0 ));
            end if;
            return Float(n);
        end Snap;

        ------------------------------------------------------------------------

        tool : A_Tool;
        temp : A_Entity;
    begin
        if first then
            if func = Primary then
                if worldX >= 0 and then worldX < world.Get_Width and then
                   worldY >= 0 and then worldY < world.Get_Height
                then
                    temp := Entities.Template( To_String( this.id ) );
                    world.Spawn_Entity( To_String( this.id ),
                                        Snap( worldX, temp.Get_Width ),
                                        Snap( worldY, temp.Get_Height ) );
                end if;
            elsif func = Secondary or else func = Tertiary then
                tool := Create_Pointer;
                Queue_Set_Tool( tool );
                Delete( tool );
            end if;
        end if;
    end Apply;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Entity_Spawner;
                         id   : String;
                         snap : Boolean ) is
    begin
        Tool(this.all).Construct;
        this.id := To_Unbounded_String( id );
        this.snap := snap;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_ID( this : not null access Entity_Spawner'Class ) return String is
    begin
        return To_String( this.id );
    end Get_ID;

    ----------------------------------------------------------------------------

    function In_Class( this    : not null access Entity_Spawner'Class;
                       pattern : String ) return Boolean is
    begin
        return Entities.In_Class( To_String( this.id ), pattern );
    end In_Class;

end Tools.Entity_Spawners;
