
private with Ada.Strings.Unbounded;

package Tools.Entity_Spawners is

    type Entity_Spawner is new Tool with private;
    type A_Entity_Spawner is access all Entity_Spawner'Class;

    -- Creates a new entity of class 'id'. If 'snap' is True, the new entity
    -- will be snapped to the grid based if the gridsnap preference is enabled
    -- and holding ctrl while using the spawner tool will reverse the gridsnap
    -- behavior. If 'snap' is False, no grid snapping will ever be performed.
    function Create_Entity_Spawner( id : String; snap : Boolean := True ) return A_Tool;
    pragma Precondition( id'Length > 0 );
    pragma Postcondition( Create_Entity_Spawner'Result /= null );

    function Get_ID( this : not null access Entity_Spawner'Class ) return String;
    pragma Postcondition( Get_ID'Result'Length > 0 );

    -- Returns True if the class id that this spawner will spawn is within
    -- the parent class specified by 'pattern' (case insensitive). For example,
    -- to check if the spawner's class id "Entities.Enemies.Cat" is an enemy,
    -- use class pattern "Entities.Enemies.*", which will match all registered
    -- enemy entity classes. A constant for matching enemy classes is specified
    -- in the Entities.Enemies package.
    function In_Class( this    : not null access Entity_Spawner'Class;
                       pattern : String ) return Boolean;

private

    use Ada.Strings.Unbounded;

    type Entity_Spawner is new Tool with
        record
            id   : Unbounded_String;
            snap : Boolean := False;    -- allow grid snapping?
        end record;

    procedure Apply( this      : access Entity_Spawner;
                     func      : Function_Type;
                     modifiers : Modifiers_Array;
                     first     : Boolean;
                     world     : not null A_World;
                     worldX,
                     worldY,
                     layer     : Integer );

    procedure Construct( this : access Entity_Spawner;
                         id   : String;
                         snap : Boolean );
    pragma Precondition( id'Length > 0 );

end Tools.Entity_Spawners;
