
--with Debugging;                         use Debugging;
with Events.Ked;                        use Events.Ked;
with Tools.Pointers;                    use Tools.Pointers;

package body Tools.Paintbrushes is

    function Create_Paintbrush( tile : Natural ) return A_Tool is
        this : constant A_Tool := new Paintbrush;
    begin
        Paintbrush(this.all).Construct( tile );
        return this;
    end Create_Paintbrush;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this      : access Paintbrush;
                     func      : Function_Type;
                     modifiers : Modifiers_Array;
                     first     : Boolean;
                     world     : not null A_World;
                     worldX,
                     worldY,
                     layer     : Integer ) is
        pragma Unreferenced( modifiers );
        tileX : constant Integer := worldX / world.Tile_Width;
        tileY : constant Integer := worldY / world.Tile_Width;
        tool  : A_Tool := null;
    begin
        if func = Primary then
            if this.lastFunc /= Primary or else
                (tileX /= this.overTileX or else tileY /= this.overTileY or else layer /= this.overLayer)
            then
                this.overTileX := tileX;
                this.overTileY := tileY;
                this.overLayer := layer;
                world.Set_Tile( layer, tileX, tileY, this.tile );
            end if;
        elsif func = Secondary then
            if this.lastFunc /= Secondary or else
                (tileX /= this.overTileX or else tileY /= this.overTileY or else layer /= this.overLayer)
            then
                this.overTileX := tileX;
                this.overTileY := tileY;
                this.overLayer := layer;
                tool := Create_Paintbrush( world.Get_Tile_Id( layer, tileX, tileY ) );
                Queue_Set_Tool( tool );
                Delete( tool );
            end if;
        elsif first and then func = Tertiary then
            tool := Create_Pointer;
            Queue_Set_Tool( tool );
            Delete( tool );
        end if;
        this.lastFunc := func;
    end Apply;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Paintbrush; tile : Natural ) is
    begin
        Object(this.all).Construct;
        this.tile := tile;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Tile( this : not null access Paintbrush'Class ) return Natural is
    begin
        return this.tile;
    end Get_Tile;

end Tools.Paintbrushes;
