
--with Debugging;                         use Debugging;
with Entities;                          use Entities;
with Entities.Items;                    use Entities.Items;
with Events.Ked;                        use Events.Ked;
with Game_Views;                        use Game_Views;
with Themes.Palette;                    use Themes.Palette;
with Tools;                             use Tools;
with Tools.Entity_Spawners;             use Tools.Entity_Spawners;

package body Widgets.Palettes.Items is

    function Create_Item_Palette( view : not null access Game_Views.Game_View'Class;
                                  id   : String ) return A_Palette is
        this : A_Palette := new Item_Palette;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Item_Palette;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Item_Palette;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is

        procedure Examine( id : String ) is
        begin
            this.itemTypes.Append( id );
        end Examine;

    begin
        Palette(this.all).Construct( view, id );
        Entities.Iterate_Classes( "Entities.Items.*", Examine'Access );
        this.Get_View.Get_Corral.Add_Listener( A_Event_Listener(this), "Set_Tool" );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Item_Palette ) is
    begin
        this.Get_View.Get_Corral.Remove_Listener( A_Event_Listener(this), "Set_Tool" );
        this.itemTypes.Clear;
        Palette(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    function Get_Item_Count( this : access Item_Palette ) return Natural is
    begin
        return Natural(this.itemTypes.Length);
    end Get_Item_Count;

    ----------------------------------------------------------------------------

    overriding
    procedure Get_Item_Details( this   : access Item_Palette;
                                row,
                                col    : Natural;
                                bmp    : out A_Bitmap;
                                border : out Color_Type ) is
        slot : constant Positive := (row - 1) * this.columns + col;
        item : A_Entity;
    begin
        bmp := null;
        if slot <= Natural(this.itemTypes.Length) then
            item := Entities.Template( this.itemTypes.Element( slot ) );
            if item /= null then
                bmp := A_Item(item).Get_Icon;
            end if;
        end if;
        border := Themes.Palette.base_colors(Transparent);
    end Get_Item_Details;

    ----------------------------------------------------------------------------

    overriding
    function Get_Scroll_Inc_Y( this : access Item_Palette ) return Integer is
    begin
        return this.iconHeight + 1;
    end Get_Scroll_Inc_Y;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Event( this : access Item_Palette;
                            evt  : in out A_Event;
                            resp : out Response_Type ) is
        pragma Unreferenced( resp );
        slot : Integer;
        tool : A_Tool;
    begin
        if evt.Get_Id = SET_TOOL_ID then
            tool := A_Set_Tool_Event(evt).Get_Tool;
            if tool.all in Entity_Spawner'Class then
                declare
                    id : constant String := A_Entity_Spawner(tool).Get_ID;
                begin
                    slot := 0;
                    for i in 1..Integer(this.itemTypes.Length) loop
                        if this.itemTypes.Element( i ) = id then
                            slot := i;
                            exit;
                        end if;
                    end loop;
                end;
            else
                slot := 0;
            end if;
            if this.selectedItem /= slot then
                this.selectedItem := slot;
                A_Widget(this).Set_Dirty;
            end if;
        end if;
    end Handle_Event;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Mouse_Press( this : access Item_Palette;
                                  evt  : not null A_Mouse_Button_Event ) is
        row  : constant Integer := (evt.Get_Y / (this.iconHeight + 1)) + 1;
        col  : constant Integer := (evt.Get_X / (this.iconWidth + 1)) + 1;
        slot : constant Integer := (row - 1) * this.columns + col;
        tool : A_Tool;
    begin
        if slot <= Natural(this.itemTypes.Length) then
            tool := Create_Entity_Spawner( this.itemTypes.Element( slot ), snap => True );
            Queue_Set_Tool( tool );
            Delete( tool );
        end if;
     end Handle_Mouse_Press;

end Widgets.Palettes.Items;
