
--with Debugging;                         use Debugging;
with Events.Ked;                        use Events.Ked;
with Events.World;                      use Events.World;
with Game_Views;                        use Game_Views;
with Physics;                           use Physics;
with Resources;                         use Resources;
with Themes.Palette;                    use Themes.Palette;
with Tools;                             use Tools;
with Tools.Paintbrushes;                use Tools.Paintbrushes;

package body Widgets.Palettes.Tiles is

    function Create_Tile_Palette( view : not null access Game_Views.Game_View'Class;
                                  id   : String ) return A_Palette is
        this : A_Palette := new Tile_Palette;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Tile_Palette;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Tile_Palette;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is
    begin
        Palette(this.all).Construct( view, id );
        this.Get_View.Get_Corral.Add_Listener( A_Event_Listener(this), "Set_Tool" );
        this.Get_View.Get_Corral.Add_Listener( A_Event_Listener(this), "World_Loaded" );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Tile_Palette ) is
    begin
        Unload_Library( this.lib );
        this.Get_View.Get_Corral.Remove_Listener( A_Event_Listener(this), "World_Loaded" );
        this.Get_View.Get_Corral.Remove_Listener( A_Event_Listener(this), "Set_Tool" );
        Palette(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    function Get_Item_Count( this : access Tile_Palette ) return Natural is
    begin
        if this.lib /= null then
            return this.lib.Get_Tile_Count;
        end if;
        return 0;
    end Get_Item_Count;

    ----------------------------------------------------------------------------

    overriding
    procedure Get_Item_Details( this   : access Tile_Palette;
                                row,
                                col    : Natural;
                                bmp    : out A_Bitmap;
                                border : out Color_Type ) is
        tile : Integer;
    begin
        bmp := null;
        border := base_colors(Transparent);
        if this.lib /= null then
            tile := this.lib.Get_Id_At_Slot( (row - 1) * this.columns + col );
            if tile /= 0 then
                bmp := this.lib.Get_Bitmap( tile );
                if bmp = null then
                    A_Widget(this).Set_Dirty;
                end if;
            end if;
            case this.lib.Get_Clip( tile ) is
                when Wall => border := Themes.Palette.base_colors(White);
                when others => border := Themes.Palette.base_colors(Transparent);
            end case;
        end if;
    end Get_Item_Details;

    ----------------------------------------------------------------------------

    overriding
    function Get_Scroll_Inc_Y( this : access Tile_Palette ) return Integer is
    begin
        return this.iconHeight + 1;
    end Get_Scroll_Inc_Y;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Event( this : access Tile_Palette;
                            evt  : in out A_Event;
                            resp : out Response_Type ) is
        pragma Unreferenced( resp );
        slot : Integer;
        tool : A_Tool;
    begin
        if evt.Get_Id = WORLD_LOADED_ID then
            this.Set_Library_Name( A_World_Loaded_Event(evt).Get_Library_Name );
        elsif evt.Get_Id = SET_TOOL_ID then
            tool := A_Set_Tool_Event(evt).Get_Tool;
            if tool.all in Paintbrush'Class then
                slot := this.lib.Get_Slot_For_ID( Paintbrush(tool.all).Get_Tile );
            else
                slot := 0;
            end if;
            if this.selectedItem /= slot then
                this.selectedItem := slot;
                A_Widget(this).Set_Dirty;
            end if;
        end if;
    end Handle_Event;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Mouse_Press( this : access Tile_Palette;
                                  evt  : not null A_Mouse_Button_Event ) is
        row     : constant Integer := (evt.Get_Y / (this.iconHeight + 1)) + 1;
        col     : constant Integer := (evt.Get_X / (this.iconWidth + 1)) + 1;
        tile_id : Integer;
        tool    : A_Tool;
    begin
        if this.lib /= null then
            tile_id := this.lib.Get_Id_At_Slot( (row - 1) * this.columns + col );
            if tile_id > 0 then
                tool := Create_Paintbrush( tile_id );
                Queue_Set_Tool( tool );
                Delete( tool );
            end if;
        end if;
     end Handle_Mouse_Press;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Library_Name( this : access Tile_Palette; name : String ) is
    begin
        if this.lib = null or else name /= this.lib.Get_Name then
            Unload_Library( this.lib );
            this.lib := Load_Library( name );
            if this.lib = null then
                raise RESOURCE_ERROR with "Error loading tile library <" & name & ">";
            end if;
            this.Scroll_To( 0, 0 );
            A_Widget(this).Set_Dirty;
            A_Widget(this).Pack;
        end if;
    end Set_Library_Name;

end Widgets.Palettes.Tiles;
