
with Ada.Strings;                       use Ada.Strings;
with Ada.Strings.Fixed;                 use Ada.Strings.Fixed;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Events.Entities;                   use Events.Entities;
with Mouse;                             use Mouse;
with Preferences;                       use Preferences;
with Support;                           use Support;
with Widgets.Containers.Panels.Dialogs.Entity_Attributes;
 use Widgets.Containers.Panels.Dialogs.Entity_Attributes;
with Widgets.Containers.Panels.Popups;  use Widgets.Containers.Panels.Popups;
with Widgets.Containers.Scenes.Ked;     use Widgets.Containers.Scenes.Ked;
with Widgets.Containers.Windows;        use Widgets.Containers.Windows;

package body Widgets.Sprites.Ked is

    DIALOG_EDIT_ATTRIBUTES : constant String := ":dlgEditAttrs";
    MENU_EDIT_ATTRIBUTES   : constant String := ":miEditAttrs";
    POPUP_MENU             : constant String := ":popup";
    GRAB_BORDER_SIZE       : constant := 4;

    ----------------------------------------------------------------------------

    function Create_Ked_Sprite( view       : not null access Game_Views.Game_View'Class;
                                eid        : Entity_Id;
                                class      : String;
                                x, y       : Float;
                                width,
                                height     : Natural;
                                resizeable : Boolean;
                                physical   : Boolean;
                                libName    : String;
                                frame      : Natural ) return A_Ked_Sprite is
        this : A_Ked_Sprite := new Ked_Sprite;
    begin
        this.Construct( view, eid, class, x, y, width, height, resizeable,
                        physical, libName, frame );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Ked_Sprite;

    ----------------------------------------------------------------------------

    not overriding
    procedure Build_Popup( this : access Ked_Sprite ) is
        popup    : A_Popup;
        menuItem : A_Menu_Item;
    begin
        if this.popupMenu = null and then this.attrs.Length > 0 then
            popup := Create_Popup( this.Get_View, this.id & POPUP_MENU );
            popup.Add_Listener( A_Visibility_Listener(this) );
            menuItem := Create_Menu_Item( this.Get_View,
                                          this.id & MENU_EDIT_ATTRIBUTES,
                                          "Edit Attributes..." );
            menuItem.Add_Listener( A_Menu_Listener(this) );
            popup.Add( A_Widget(menuItem) );
            this.popupMenu := A_Widget(popup);
        end if;
    end Build_Popup;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this       : access Ked_Sprite;
                         view       : not null access Game_Views.Game_View'Class;
                         eid        : Entity_Id;
                         class      : String;
                         x, y       : Float;
                         width,
                         height     : Natural;
                         resizeable : Boolean;
                         physical   : Boolean;
                         libName    : String;
                         frame      : Natural ) is
    begin
        Sprite(this.all).Construct( view, eid, class, x, y, physical, libName, frame );
        this.interactive := True;
        this.resizeable := resizeable;
        if this.resizeable then
            this.border := Border_Line;
        else
            this.snapCenter := True;    -- snap center on items, enemies, etc.
        end if;

        -- position the sprite within its parent
        declare
            w : constant Integer := Integer'Max( this.Get_Min_Width, width );
            h : constant Integer := Integer'Max( this.Get_Min_Height, height );
        begin
            this.Set_Layout_XYWH( Integer(this.x) - w / 2,
                                  Integer(this.y) - h / 2,
                                  w, h );
        end;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Ked_Sprite ) is
    begin
        if this.popupMenu /= null then
            this.popupMenu.Remove_Listener( A_Visibility_Listener(this) );
            Delete( this.popupMenu );
        end if;
        if this.dialog /= null then
            this.dialog.Remove_Listener( A_Visibility_Listener(this) );
        end if;
        Sprite(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Ked_Sprite;
                             action : A_Menu_Action ) is

        function Short_Class( class : Unbounded_String ) return String is
            class2 : constant String := To_String( class );
            dot    : constant Integer := Index( class2, ".", Backward );
        begin
            if dot >= class2'First then
                return class2(dot+1..class2'Last);
            end if;
            return To_String( class );
        end Short_Class;

    begin
        if action.Get_Id = Menu_Selected then
            if action.Get_Source.Get_Id = To_String( this.id & MENU_EDIT_ATTRIBUTES ) then
                if this.dialog /= null then
                    this.Get_Window.Remove( A_Widget(this.dialog) );
                    Delete( A_Widget(this.dialog) );
                end if;
                this.dialog := Create_Entity_Attributes_Dialog( this.Get_View,
                    this.id & DIALOG_EDIT_ATTRIBUTES, this.eid,
                    Short_Class( this.class ), this.attrs );
                this.dialog.Add_Listener( A_Visibility_Listener(this) );
                this.Get_Window.Add( A_Widget(this.dialog), consume => False );
                this.dialog.Show;
            end if;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Ked_Sprite;
                             action : A_Visibility_Action ) is
    begin
        if action.Get_Id = Change_Visibility then
            if action.Get_Source.Get_Id = To_String( this.id & POPUP_MENU ) then
                if action.Is_Visible then
                    pragma Debug( Dbg( To_String( this ) & " popup is visible", D_GUI, INFO ) );
                    null;
                else
                    pragma Debug( Dbg( To_String( this ) & " popup is hidden", D_GUI, INFO ) );
                    this.updatable := this.updatable - 1;
                end if;
            elsif action.Get_Source.Get_Id = To_String( this.id & DIALOG_EDIT_ATTRIBUTES ) then
                if action.Is_Visible then
                    pragma Debug( Dbg( To_String( this ) & " dialog is visible", D_GUI, INFO ) );
                    null;
                else
                    pragma Debug( Dbg( To_String( this ) & " dialog is hidden", D_GUI, INFO ) );
                    null;
                end if;
            end if;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Focus( this : access Ked_Sprite ) is
    begin
        this.Bring_To_Front;
    end Handle_Focus;

    ----------------------------------------------------------------------------

    overriding
    function Handle_Key_Held( this : access Ked_Sprite;
                              evt  : not null A_Key_Event ) return Boolean is
    begin
        if (evt.Get_Key = KEY_LEFT or else
            evt.Get_Key = KEY_RIGHT or else
            evt.Get_Key = KEY_UP or else
            evt.Get_Key = KEY_DOWN) and then
           this.physical
        then
            return True;
        elsif evt.Get_Key = KEY_DEL and then evt.No_Modifiers then
            return True;
        else
            return Sprite(this.all).Handle_Key_Held( evt );
        end if;
    end Handle_Key_Held;

    ----------------------------------------------------------------------------

    overriding
    function Handle_Key_Press( this : access Ked_Sprite;
                               evt  : not null A_Key_Event ) return Boolean is
    begin
        if evt.Get_Key = KEY_DEL and then evt.No_Modifiers then
            Queue_Delete_Entity( this.eid );
            return True;

        elsif evt.Get_Key = KEY_LEFT and then this.physical then
            Queue_Entity_Face( this.eid, Dir_Left );
            return True;
        elsif evt.Get_Key = KEY_RIGHT and then this.physical then
            Queue_Entity_Face( this.eid, Dir_Right );
            return True;
        elsif evt.Get_Key = KEY_UP and then this.physical then
            Queue_Entity_Face( this.eid, Dir_Up );
            return True;
        elsif evt.Get_Key = KEY_DOWN and then this.physical then
            Queue_Entity_Face( this.eid, Dir_Down );
            return True;

        else
            return Sprite(this.all).Handle_Key_Press( evt );
        end if;
    end Handle_Key_Press;

    ----------------------------------------------------------------------------

    overriding
    function Handle_Key_Release( this : access Ked_Sprite;
                                 evt  : not null A_Key_Event ) return Boolean is
    begin
        if (evt.Get_Key = KEY_LEFT or else
            evt.Get_Key = KEY_RIGHT or else
            evt.Get_Key = KEY_UP or else
            evt.Get_Key = KEY_DOWN) and then
           this.physical
        then
            return True;

        elsif evt.Get_Key = KEY_DEL and then evt.No_Modifiers then
            return True;

        else
            return Sprite(this.all).Handle_Key_Release( evt );
        end if;
    end Handle_Key_Release;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Mouse_Move( this : access Ked_Sprite;
                                 evt  : not null A_Mouse_Event ) is
        ex : constant Integer := evt.Get_X;
        ey : constant Integer := evt.Get_Y;
    begin
        Sprite(this.all).Handle_Mouse_Move( evt );
        if this.pressed and then
           (ex /= this.dragStartX or else ey /= this.dragStartY)
        then
            case this.dragAction is
                when Drag_None => null;
                when Drag_Move =>
                    this.Move( ex - this.dragStartX, ey - this.dragStartY );
                    this.dragged := True;
                when Drag_Resize =>
                    this.Resize( this.resizeDir, ex - this.dragStartX, ey - this.dragStartY );
            end case;
        end if;
    end Handle_Mouse_Move;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Mouse_Press( this : access Ked_Sprite;
                                  evt  : not null A_Mouse_Button_Event ) is
    begin
        Sprite(this.all).Handle_Mouse_Press( evt );
        if evt.Get_Button = Mouse_Left then
            this.dragStartX := evt.Get_X;
            this.dragStartY := evt.Get_Y;
            this.dragged := False;
            if this.resizeable then
                if Contains( GRAB_BORDER_SIZE, 0,
                             this.cwidth - GRAB_BORDER_SIZE - 1, GRAB_BORDER_SIZE - 1,
                             this.dragStartX, this.dragStartY )
                then
                    -- grabbed top edge
                    this.dragAction := Drag_Resize;
                    this.resizeDir := Dir_Up;
                elsif Contains( GRAB_BORDER_SIZE, this.cheight - GRAB_BORDER_SIZE,
                                this.cwidth - GRAB_BORDER_SIZE - 1, this.cheight - 1,
                                this.dragStartX, this.dragStartY )
                then
                    -- grabbed bottom edge
                    this.dragAction := Drag_Resize;
                    this.resizeDir := Dir_Down;
                elsif Contains( 0, GRAB_BORDER_SIZE,
                                GRAB_BORDER_SIZE - 1, this.cheight - GRAB_BORDER_SIZE - 1,
                                this.dragStartX, this.dragStartY )
                then
                    -- grabbed left edge
                    this.dragAction := Drag_Resize;
                    this.resizeDir := Dir_Left;
                elsif Contains( this.cwidth - GRAB_BORDER_SIZE, GRAB_BORDER_SIZE,
                                this.cwidth - 1, this.cheight - GRAB_BORDER_SIZE - 1,
                                this.dragStartX, this.dragStartY )
                then
                    -- grabbed right edge
                    this.dragAction := Drag_Resize;
                    this.resizeDir := Dir_Right;
                elsif Contains( 0, 0, GRAB_BORDER_SIZE - 1, GRAB_BORDER_SIZE - 1,
                                this.dragStartX, this.dragStartY )
                then
                    -- grabbed top left corner
                    this.dragAction := Drag_Resize;
                    this.resizeDir := Dir_Up_Left;
                elsif Contains( this.cwidth - GRAB_BORDER_SIZE, 0,
                                this.cwidth - 1, GRAB_BORDER_SIZE - 1,
                                this.dragStartX, this.dragStartY )
                then
                    -- grabbed top right corner
                    this.dragAction := Drag_Resize;
                    this.resizeDir := Dir_Up_Right;
                elsif Contains( 0, this.cheight - GRAB_BORDER_SIZE,
                                GRAB_BORDER_SIZE - 1, this.cheight - 1,
                                this.dragStartX, this.dragStartY )
                then
                    -- grabbed bottom left corner
                    this.dragAction := Drag_Resize;
                    this.resizeDir := Dir_Down_Left;
                elsif Contains( this.cwidth - GRAB_BORDER_SIZE, this.cheight - GRAB_BORDER_SIZE,
                                this.cwidth - 1, this.cheight - 1,
                                this.dragStartX, this.dragStartY )
                then
                    -- grabbed bottom right corner
                    this.dragAction := Drag_Resize;
                    this.resizeDir := Dir_Down_Right;
                else
                    this.dragAction := Drag_Move;
                end if;
            else
                this.dragAction := Drag_Move;
            end if;
        end if;
        this.updatable := this.updatable + 1;
        A_Ked_Scene(this.parent).Set_Selection( A_Widget(this) );
    end Handle_Mouse_Press;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Mouse_Release( this : access Ked_Sprite;
                                    evt  : not null A_Mouse_Button_Event ) is
        gs : constant Positive := Get_Pref( "gridsize" );

        ------------------------------------------------------------------------

        function Snap( n : Integer ) return Float is
        begin
            if not this.physical and then Get_Pref( "gridsnap" ) then
                return Float(Support.Grid_Snap( n, gs, this.snapCenter ));
            end if;
            return Float(n);
        end Snap;

        ------------------------------------------------------------------------

    begin
        Sprite(this.all).Handle_Mouse_Release( evt );
        if evt.Get_Button = Mouse_Left then
            this.updatable := this.updatable - 1;
            case this.dragAction is
                when Drag_Move =>
                    if this.dragged then
                        if this.snapCenter then
                            -- snap the center of the sprite to the center of
                            -- the grid
                            Queue_Move_Entity( this.eid,
                                               Snap( this.x1 + this.Get_Width / 2 ),
                                               Snap( this.y1 + this.Get_Height / 2 ) );
                        else
                            -- snap the top left corner to the grid. this should
                            -- be done for resizeable sprites.
                            Queue_Move_Entity( this.eid,
                                               Snap( this.x1 ) + this.Get_Width / 2,
                                               Snap( this.y1 ) + this.Get_Height / 2 );
                        end if;
                    end if;
                when Drag_Resize =>
                    -- snapping occurs during the resize so it's already done
                    Queue_Resize_Entity( this.eid, this.Get_Width, this.Get_Height );
                    Queue_Move_Entity( this.eid,
                                       Float(this.x1 + this.Get_Width / 2),
                                       Float(this.y1 + this.Get_Height / 2) );
                when others => null;
            end case;
            this.dragAction := Drag_None;
        elsif evt.Get_Button = Mouse_Right then
            this.Build_Popup;
            if this.popupMenu /= null then
                this.Activate_Popup( this.popupMenu, Widget_Top_Left,
                                     evt.Get_X, evt.Get_Y, 0, 0, False );
            else
                this.updatable := this.updatable - 1;
            end if;
        elsif evt.Get_Button = Mouse_Middle then
            this.updatable := this.updatable - 1;
        end if;
    end Handle_Mouse_Release;

    ----------------------------------------------------------------------------

    not overriding
    procedure Resize( this : access Ked_Sprite;
                      dir  : Direction_Type;
                      xAmt,
                      yAmt : Integer ) is

        ------------------------------------------------------------------------

        function Snap_Adjusted( n : Integer ) return Integer is
            gridSize : constant Positive := Get_Pref( "gridsize" );
            amount   : Integer := 0;
        begin
            if Get_Pref( "gridsnap" ) then
                amount := n mod gridSize;
                if amount <= 7 then
                    amount := -amount;
                else
                    amount := gridSize - amount;
                end if;
            end if;
            return amount + n;
        end Snap_Adjusted;

        ------------------------------------------------------------------------

        left    : Integer := this.x1;
        top     : Integer := this.y1;
        width   : Integer := this.cwidth;
        height  : Integer := this.cheight;
        resized : Boolean := False;
        right   : Integer;
        bottom  : Integer;
        newPos  : Integer;
    begin
        if dir.x < 0 then
            -- move left edge
            newPos := Snap_Adjusted( left + xAmt );
            if newPos >= 0 and then
               (newPos < left or else width - (newPos - left) >= this.Get_Min_Width)
            then
                width := width - (newPos - left);
                left := newPos;
                resized := True;
            end if;
        elsif dir.x > 0 then
            -- move right edge
            right := left + this.cwidth - 1;
            newPos := Snap_Adjusted( right + xAmt ) - 1;
            if newPos > right or else width - (right - newPos) >= this.Get_Min_Width then
                width := width + (newPos - right);
                this.dragStartX := this.dragStartX + (newPos - right);
                resized := True;
            end if;
        end if;
        if dir.y < 0 then
            -- move top
            newPos := Snap_Adjusted( top + yAmt );
            if newPos >= 0 and then
               (newPos < top or else height - (newPos - top) >= this.Get_Min_Height)
            then
                height := height - (newPos - top);
                top := newPos;
                resized := True;
            end if;
        elsif dir.y > 0 then
            -- move bottom
            bottom := top + this.cheight - 1;
            newPos := Snap_Adjusted( bottom + yAmt ) - 1;
            if newPos > bottom or else height - (bottom - newPos) >= this.Get_Min_Height then
                height := height + (newPos - bottom);
                this.dragStartY := this.dragStartY + (newPos - bottom);
                resized := True;
            end if;
        end if;
        if resized then
            this.Set_Layout_LTWH( left, top, width, height );
        end if;
    end Resize;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Attributes( this       : access Ked_Sprite;
                              attributes : in out A_Association ) is
    begin
        Delete( this.attrs );
        this.attrs := attributes;
        attributes := null;
        A_Widget(this).Set_Dirty;
    end Set_Attributes;

end Widgets.Sprites.Ked;
