
with Actors;
with Ada.Tags;                          use Ada.Tags;
--with Debugging;                         use Debugging;
with Directions;                        use Directions;
with Entities.Players;                  use Entities.Players;
with Entities.Players.Keen;             use Entities.Players.Keen;
with Entities.Players.Little_Keen;      use Entities.Players.Little_Keen;
with Games;                             use Games;
with Impulses;                          use Impulses;

package body Actors.Keen is

    function Create_Keen return A_Actor is
        this : constant A_Keen_Actor := new Keen_Actor;
    begin
        this.Construct;
        return A_Actor(this);
    end Create_Keen;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Keen_Actor ) is
    begin
        Actor(this.all).Construct;
        this.ammo := 5;
        this.drops := 0;
        this.points := 0;
        this.nextLife := 20_000;
        this.lives := 3;
        this.ancients := 0;
        this.scuba := False;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Impulse( this : access Keen_Actor; name : Hashed_String ) is
    begin
        if this.entity /= null then
            this.behavior.Impulse( name );
        end if;
    end Impulse;

    ----------------------------------------------------------------------------

    overriding
    function Is_Temporal( this : access Keen_Actor ) return Boolean is
        pragma Unreferenced( this );
    begin
        return False;
    end Is_Temporal;

    ----------------------------------------------------------------------------

    overriding
    procedure Pause( this : access Keen_Actor ) is
    begin
        this.behavior.Pause;
    end Pause;

    ----------------------------------------------------------------------------

    overriding
    function Set_Entity( this : access Keen_Actor; e : A_Entity ) return Boolean is
    begin
        if e = null then
            this.Set_Entity( e );
            Delete( A_Limited_Object(this.behavior) );
            return True;

        elsif e.all'Tag = Entities.Players.Keen.Keen'Tag then
            this.Set_Entity( e );
            Delete( A_Limited_Object(this.behavior) );
            this.behavior := new Platform_Behavior;
            this.behavior.actor := A_Keen_Actor(this);
            return True;

        elsif e.all'Tag = Entities.Players.Little_Keen.Little_Keen'Tag then
            this.Set_Entity( e );
            Delete( A_Limited_Object(this.behavior) );
            this.behavior := new Map_Behavior;
            this.behavior.actor := A_Keen_Actor(this);
            return True;

        end if;
        return False;
    end Set_Entity;

    --==========================================================================

    overriding
    procedure Impulse( this : access Map_Behavior; name : Hashed_String ) is
        keen : constant A_Little_Keen := A_Little_Keen(this.actor.entity);
    begin
        if name = Shoot or else name = Jump_Start then
            declare
                procedure Activate( e : A_Entity ) is
                begin
                    e.Activate( activator => this.actor.entity );
                end Activate;
            begin
                keen.Iterate_Touching( Activate'Access );
            end;

        elsif name = Move_Left_Start then
            keen.Move_Start( Left );

        elsif name = Move_Left_Stop then
            keen.Move_Stop( Left );

        elsif name = Move_Right_Start then
            keen.Move_Start( Right );

        elsif name = Move_Right_Stop then
            keen.Move_Stop( Right );

        elsif name = Move_Up_Start then
            keen.Move_Start( Up );

        elsif name = Move_Up_Stop then
            keen.Move_Stop( Up );

        elsif name = Move_Down_Start then
            keen.Move_Start( Down );

        elsif name = Move_Down_Stop then
            keen.Move_Stop( Down );

        end if;
    end Impulse;

    ----------------------------------------------------------------------------

    overriding
    procedure Pause( this : access Map_Behavior ) is
        keen : constant A_Little_Keen := A_Little_Keen(this.actor.entity);
    begin
        keen.Move_Stop( Left );
        keen.Move_Stop( Right );
        keen.Move_Stop( Up );
        keen.Move_Stop( Down );
    end Pause;

    --==========================================================================

    overriding
    procedure Impulse( this : access Platform_Behavior; name : Hashed_String ) is
        keen : constant A_Keen := A_Keen(this.actor.entity);
    begin
        if name = Move_Left_Start then
            keen.Move_Start( Left );

        elsif name = Move_Left_Stop then
            keen.Move_Stop( Left );

        elsif name = Move_Right_Start then
            keen.Move_Start( Right );

        elsif name = Move_Right_Stop then
            keen.Move_Stop( Right );

        elsif name = Move_Up_Start then
            -- keen is trying to activate entities, grab a pole, or look up
            keen.Try_To_Activate;
            keen.Try_To_Climb( Up );
            keen.Move_Start( Up );

        elsif name = Move_Up_Stop then
            -- keen is no longer trying to activate entities or look up
            keen.Stop_Activating;
            keen.Move_Stop( Up );

        elsif name = Move_Down_Start then
            keen.Try_To_Climb( Down );
            keen.Move_Start( Down );

        elsif name = Move_Down_Stop then
            keen.Move_Stop( Down );

        elsif name = Jump_Start then
            -- keen is attempting to jump
            keen.Try_To_Jump;

        elsif name = Jump_Stop then
            -- keen is no longer attempting to jump
            keen.Stop_Jump_Attempt;

        elsif name = Toggle_Pogo then
            -- toggle the pogostick on or off if possible
            keen.Toggle_Pogostick;

        elsif name = Shoot then
            if this.actor.game.Get_Game_Var( "ammo" ) > 0 then
                if keen.Shoot_Gun( fire => True ) then
                    this.actor.game.Game_Var_Add( "ammo", -1 );
                end if;
            else
                if keen.Shoot_Gun( fire => False ) then
                    null;
                end if;
            end if;

        elsif name = Finish_Teleporting then
            keen.Finish_Teleport;

        end if;
    end Impulse;

    ----------------------------------------------------------------------------

    overriding
    procedure Pause( this : access Platform_Behavior ) is
        keen : constant A_Keen := A_Keen(this.actor.entity);
    begin
        keen.Move_Stop( Left );
        keen.Move_Stop( Right );
        keen.Move_Stop( Up );
        keen.Move_Stop( Down );
        keen.Stop_Jump_Attempt;
    end Pause;

end Actors.Keen;
