
private with Events.Entities;
private with Events.Keen;
private with Events.Game;

package Games.Keen is

    pragma Elaborate_Body;

private

    use Events.Entities;
    use Events.Game;
    use Events.Keen;

    type Keen_Game is new Game with
        record
            dialogId   : Integer := Integer'First;    -- current open dialog
            mapWorld   : A_World;
            playingMap : Boolean := True;    -- playing the overhead map
        end record;

    procedure Add_Event_Listeners( this : access Keen_Game );

    procedure Delete( this : in out Keen_Game );

    procedure End_Game( this : access Keen_Game );

    -- Enters a level, raising an exception on error loading world 'name'.
    procedure Enter_Level( this : not null access Keen_Game'Class;
                           name : String );

    -- Exits a level, returning to the overhead map. If 'success' is True, the
    -- current level will be marked as complete in the game session. An
    -- exception will be raised on error loading the world.
    procedure Exit_Level( this    : not null access Keen_Game'Class;
                          success : Boolean );

    procedure Handle_Event( this : access Keen_Game;
                            evt  : in out A_Event;
                            resp : out Response_Type );
    pragma Precondition( evt /= null );

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Dialog_Response_Event );

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Entity_Entered_Tile_Event );

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Give_Ammo_Event );

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Give_Drops_Event );

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Give_Points_Event );

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Impulse_Event );

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Player_Died_Event );

    procedure Handle( this : not null access Keen_Game'Class;
                      evt  : not null A_Set_Tile_Event );

    -- Called when the Load_World event is received, to load the next level. If
    -- the world name to load is an empty string, the current level will be
    -- exited, marked as complete, and the overhead map will be loaded. No
    -- exception will be raised on error; the error message will be put into the
    -- response 'resp'.
    procedure Handle_Load_World( this : access Keen_Game;
                                 evt  : not null A_Load_World_Event;
                                 resp : out Response_Type );

    procedure New_Game( this : access Keen_Game );

    procedure One_Up( this : not null access Keen_Game'Class );

    procedure Pause( this : access Keen_Game; enabled : Boolean );

    procedure Remove_Event_Listeners( this : access Keen_Game );

    -- Reloads the current level. An exception will be raised on error loading
    -- the world.
    procedure Retry_Level( this : not null access Keen_Game'Class );

end Games.Keen;
