
with Ada.Tags;                          use Ada.Tags;
with Entities;
with Resources;                         use Resources;

pragma Elaborate_All( Entities );

package body Entities.Poles is

    DEFAULT_HEIGHT : constant := 32;
    DEFAULT_WIDTH  : constant := 16;
    POLE_LIBRARY   : constant String := "icons";
    POLE_ICON      : constant String := "pole_entity.bmp";

    ----------------------------------------------------------------------------

    function Create_Pole return A_Entity is
        this : A_Entity := new Pole;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Pole;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Pole ) is
    begin
        Entity(this.all).Construct( DEFAULT_WIDTH, DEFAULT_HEIGHT, POLE_LIBRARY );
        this.frame := this.lib.Get_Id( POLE_ICON );
        if this.frame = 0 then
            raise RESOURCE_ERROR with "Image <" & POLE_ICON & "> not found in " &
                                      "tile library <" & POLE_LIBRARY & ">";
        end if;
        this.metaphysical := True;
        this.physical := False;
        this.clipped := False;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Pole is
        this : aliased Pole;
    begin
        this.Construct;
        Pole'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Pole ) is
    begin
        Entity'Read( stream, Entity(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Pole ) is
    begin
        Entity'Write( stream, Entity(obj) );
    end Object_Write;

begin

    Entities.Factory.Register_Class( External_Tag( Pole'Tag ), Create_Pole'Access );

end Entities.Poles;
